//////////////////////////////////////////////////////////////////////////////
// Quadratic Function Evaluation
//
// Last update: 25/08/2008
//////////////////////////////////////////////////////////////////////////////

import java.applet.Applet;
import java.math.BigDecimal;
import java.math.BigInteger;

import jscl.math.Expression;
import jscl.math.JSCLInteger;
import jscl.text.ParseException;
import javax.swing.JOptionPane;
import java.util.regex.*;


public class QuadraticFunctionEvaluation extends Applet
{
   public BigDecimal zero_bd = new BigDecimal("0");
   public BigDecimal a_bd = new BigDecimal("1");
   public BigDecimal b_bd = new BigDecimal("0");
   public BigDecimal c_bd = new BigDecimal("0");
   public BigDecimal delta_bd = new BigDecimal("0");

   public Double delta_d = new Double("0");
   
   public String a_str;
   public String b_str;
   public String c_str;
   public String d_str;
   public String delta_str;
   public String xv_str;
   public String yv_str;
   public String yf_str;
   public String ff_b_div_a_str;
   public String ff_b2_div_4a2_str;
   public String ff_m_b2_div_4a_str;
   public String ff_b_div_2a_str;
   public String ff_m_b2_div_4a_p_c_str;
   public String rt_sqrt_delta_str;  
   public String rr_r1_str = "";
   public String rr_r2_str = "";
   public String rr_with_sqrt_num_of_m_b_div_2a_str;
   public String rr_with_sqrt_den_of_m_b_div_2a_str;
   public String rr_with_sqrt_p_str; // sqrt(delta) = p*sqrt(q), where sqrt(q) cannot be simplified
   public String rr_with_sqrt_q_str;
   public String rr_with_sqrt_num_of_p_div_2a_str; 
   public String rr_with_sqrt_den_of_p_div_2a_str;
   
   public boolean realRoots = true;
   public int a_sign = 1;
   public int b_sign = 0;
   public int c_sign = 0;
   public int delta_sign = 0;
   
   ////////////////////////////////////////////////////////////////////////////
   // This procedure is executed when the applet starts
   ////////////////////////////////////////////////////////////////////////////   
   public void init()
   {	   	   
	  this.updateCoefficients("1", "0", "0");
	  
	  
	  super.init();            
   } // End of init()
   

   ////////////////////////////////////////////////////////////////////////////
   // Compure p in sqrt(delta) = p sqrt(q) where sqrt(q) cannot be simplified
   ////////////////////////////////////////////////////////////////////////////   
   public String computeP(String s)
   {
	   String p = "1";
       String t = s;
       String b = "";
       String a = "";
      
	   int n;
	   
	   n = t.indexOf("*");
	   while (n != -1)
	   {
		   b = t.substring(0, n);
		   a = t.substring(n + 1, t.length());
		   if (b.indexOf("sqrt") == -1)
		   {
			   p = p + "*" + b;
		   }
		   t = a;
		   n = t.indexOf("*");
	   }
	   if ((t.indexOf("sqrt") == -1) && (a.length() > 0))
	   {
		   p = p + "*" + t;
	   }
	   
	   return(p);
   } // End of computeP()
   
   
   ////////////////////////////////////////////////////////////////////////////
   // Compute quadratic function for a give value of x
   ////////////////////////////////////////////////////////////////////////////
   public String compute(String x_str)
   {
        BigDecimal x_bd = new BigDecimal(x_str);
        BigDecimal y_bd = new BigDecimal(x_str);
	    y_bd = y_bd.multiply(a_bd);
	    y_bd = y_bd.add(b_bd);
	    y_bd = y_bd.multiply(x_bd);
	    y_bd = y_bd.add(c_bd);
	    return(y_bd.toString());
   } // End of compute()

   
   ////////////////////////////////////////////////////////////////////////////
   // Update all elements for new coefficients
   ////////////////////////////////////////////////////////////////////////////   
   public void updateCoefficients(String input_a_str, String input_b_str, String input_c_str)
   {
	  a_str = input_a_str;
	  b_str = input_b_str;
	  c_str = input_c_str;	  
      a_bd = new BigDecimal(a_str);
      b_bd = new BigDecimal(b_str);
      c_bd = new BigDecimal(c_str);
      
      a_sign = a_bd.compareTo(zero_bd);
      b_sign = b_bd.compareTo(zero_bd);
      c_sign = c_bd.compareTo(zero_bd);      
      
      try
      {  
    	 // Delta
         String delta = "(" + b_str + ")^2 - 4*(" + a_str + ")*(" + c_str + ")";
         delta_str = Expression.valueOf(delta).expand().toString();
         
         delta_bd = new BigDecimal(delta_str);
         delta_sign = delta_bd.compareTo(zero_bd);                  
         
         // Standard form
		 String ff_b_div_a = "(" + b_str + ")/(" + a_str + ")";
		 ff_b_div_a_str = Expression.valueOf(ff_b_div_a).simplify().toString();
		 
		 String ff_b2_div_4a2 = "((" + b_str + ")^2)/(4*(" + a_str + ")^2)";
		 ff_b2_div_4a2_str = Expression.valueOf(ff_b2_div_4a2).simplify().toString();
		 
		 String ff_m_b2_div_4a = "(-(" + b_str + ")^2)/(4*(" + a_str + "))";
		 ff_m_b2_div_4a_str = Expression.valueOf(ff_m_b2_div_4a).simplify().toString(); 
		 
		 String ff_b_div_2a = "((" + b_str + "))/(2*(" + a_str + "))";
		 ff_b_div_2a_str = Expression.valueOf(ff_b_div_2a).simplify().toString(); 
		 
		 String ff_m_b2_div_4a_p_c = "(-(" + b_str + ")^2 + 4*(" + a_str + ")*(" + c_str + "))/(4*(" + a_str + "))";
		 ff_m_b2_div_4a_p_c_str = Expression.valueOf(ff_m_b2_div_4a_p_c).simplify().toString();
		 
		 // Roots
		 if (delta_sign >= 0)
		 {		 
			String rt_sqrt_delta = "sqrt((" + b_str + ")^2 - 4*(" + a_str + ")*(" + c_str + "))";  
			rt_sqrt_delta_str = Expression.valueOf(rt_sqrt_delta).simplify().toString();

			String rr_r1 = "(-(" + b_str + ") - sqrt(" + delta_str + "))/(2*("+ a_str + "))"; 
			rr_r1_str = Expression.valueOf(rr_r1).simplify().toString(); 
			   
			String rr_r2 = "(-(" + b_str + ") + sqrt(" + delta_str + "))/(2*("+ a_str + "))";
			rr_r2_str = Expression.valueOf(rr_r2).simplify().toString();

			
			
			if (rt_sqrt_delta_str.indexOf("sqrt") != -1)
			{		
				String rr_with_sqrt_num_of_m_b_div_2a;
				String rr_with_sqrt_den_of_m_b_div_2a;
				String rr_with_sqrt_p; // sqrt(delta) = p*sqrt(q), where sqrt(q) cannot be simplified
				String rr_with_sqrt_q;
				String rr_with_sqrt_num_of_p_div_2a; 
				String rr_with_sqrt_den_of_p_div_2a;
				

				// Compute p and q
				rr_with_sqrt_p = computeP(rt_sqrt_delta_str);
				rr_with_sqrt_p_str = Expression.valueOf(rr_with_sqrt_p).expand().toString();
				rr_with_sqrt_q = delta_str + "/((" + rr_with_sqrt_p_str + ")*(" + rr_with_sqrt_p_str + "))";
				rr_with_sqrt_q_str = Expression.valueOf(rr_with_sqrt_q).simplify().toString();
				
				String rr_with_sqrt_p_div_2a = rr_with_sqrt_p_str + "/(2*(" + a_str + "))";
				String rr_with_sqrt_p_div_2a_str = Expression.valueOf(rr_with_sqrt_p_div_2a).simplify().toString();
				
				int index = rr_with_sqrt_p_div_2a_str.indexOf("/");
				if (index == -1)
				{
					rr_with_sqrt_num_of_p_div_2a = rr_with_sqrt_p_div_2a_str;
					rr_with_sqrt_den_of_p_div_2a = "1";
				}
				else
				{
					rr_with_sqrt_num_of_p_div_2a = rr_with_sqrt_p_div_2a_str.substring(0, index);
					rr_with_sqrt_den_of_p_div_2a = rr_with_sqrt_p_div_2a_str.substring(index + 1, rr_with_sqrt_p_div_2a_str.length());					
				}

				String rr_with_sqrt_m_b_div_2a = "-(" + b_str + ")/(2*(" + a_str + "))";
				String rr_with_sqrt_m_b_div_2a_str = Expression.valueOf(rr_with_sqrt_m_b_div_2a).simplify().toString();
				index = rr_with_sqrt_m_b_div_2a_str.indexOf("/");
				if (index == -1)
				{
					rr_with_sqrt_num_of_m_b_div_2a = rr_with_sqrt_m_b_div_2a_str;
					rr_with_sqrt_den_of_m_b_div_2a = "1";
				}
				else
				{
					rr_with_sqrt_num_of_m_b_div_2a = rr_with_sqrt_m_b_div_2a_str.substring(0, index);
					rr_with_sqrt_den_of_m_b_div_2a = rr_with_sqrt_m_b_div_2a_str.substring(index + 1, rr_with_sqrt_m_b_div_2a_str.length());					
				}

				JSCLInteger den_of_m_b_div_2a = new JSCLInteger(new BigInteger(rr_with_sqrt_den_of_m_b_div_2a));
				JSCLInteger den_of_p_div_2a = new JSCLInteger(new BigInteger(rr_with_sqrt_den_of_p_div_2a));
				
				String gcd = den_of_m_b_div_2a.gcd(den_of_p_div_2a).toString();
				String lcm = "(" + rr_with_sqrt_den_of_p_div_2a + "*" + rr_with_sqrt_den_of_m_b_div_2a + ")/" + gcd;
				lcm = Expression.valueOf(lcm).expand().toString();				
								
				rr_with_sqrt_num_of_m_b_div_2a_str = "(" + rr_with_sqrt_num_of_m_b_div_2a + ")*" + lcm + "/" + rr_with_sqrt_den_of_m_b_div_2a; 
				rr_with_sqrt_num_of_p_div_2a_str = rr_with_sqrt_num_of_p_div_2a + "*" + lcm + "/" + rr_with_sqrt_den_of_p_div_2a;
				
				rr_with_sqrt_num_of_m_b_div_2a_str = Expression.valueOf(rr_with_sqrt_num_of_m_b_div_2a_str).simplify().toString();
				rr_with_sqrt_num_of_p_div_2a_str = Expression.valueOf(rr_with_sqrt_num_of_p_div_2a_str).simplify().toString(); 
				rr_with_sqrt_den_of_m_b_div_2a_str = lcm;
				rr_with_sqrt_den_of_p_div_2a_str = lcm;
			};
		 }		    
         
		 
         // Vertex
         String xv = "-(" + b_str + ")/(2*" + a_str + ")";
         xv_str = Expression.valueOf(xv).simplify().toString();
         
         String yv = "-(((" + b_str + ")^2) - 4*(" + a_str + ")*(" + c_str + "))/(4*" + a_str + ")";
         yv_str = Expression.valueOf(yv).simplify().toString();

         // Focus
         String yf = "(-(((" + b_str + ")^2) - 4*(" + a_str + ")*(" + c_str + ")) + 1)/(4*" + a_str + ")";
         yf_str = Expression.valueOf(yf).simplify().toString();
         
         // Directrix
         String d = "(-(((" + b_str + ")^2) - 4*(" + a_str + ")*(" + c_str + ")) - 1)/(4*" + a_str + ")";
         d_str = Expression.valueOf(d).simplify().toString();     
                  
      }
          catch(ParseException e)
      {
    	  System.out.println("Parser problem!");
      }      
   } // End of updateCoefficients() 
} // End of QuadraticFunctionEvaluation
